#!/usr/bin/python
# Copyright 2015 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""Creates a list of supported smart card reader descriptions in a
format suitable for using in the Chrome App manifest (as the value of
the $.permissions.[?(@.usbDevices)] JSON element)."""

import sys

items = []
ignored_item_count = 0
current_section = None
for line in sys.stdin:
    if not line.strip():
        continue
    if (line.startswith('# section:')):
        current_section = line[len('# section:'):].strip()
    if line.startswith('#'):
        continue
    parts = line.split(':')
    vendor_id = int(parts[0], 16)
    product_id = int(parts[1], 16)
    if current_section in ('supported', 'shouldwork'):
        items.append((vendor_id, product_id))
    else:
        ignored_item_count += 1

print ',\n'.join(
    '        {{"vendorId": {0}, "productId": {1}}}'.format(
        vendor_id, product_id)
    for vendor_id, product_id in sorted(items))

print >>sys.stderr, ('Extracted {0} supported USB devices and ignored {1} '
                     'items.'.format(len(items), ignored_item_count))
