# Copyright 2015 The Chromium Authors. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.


#
# This file contains helper definitions for using this library.
#
# /common/make/common.mk must be included before including this file.
#


CLOSURE_LIBRARY_VERSION := 20160208


CLOSURE_LIBRARY_RELATIVE_DIR_PATH := \
	closure-library/src-$(CLOSURE_LIBRARY_VERSION)

CLOSURE_LIBRARY_DIR_PATH := \
	$(THIRD_PARTY_DIR_PATH)/$(CLOSURE_LIBRARY_RELATIVE_DIR_PATH)


#
# List of all source files and directories in the Closure Library.
#
# This variable is intended to be used as make rules prerequisites.
#

CLOSURE_LIBRARY_SOURCES_AND_DIRS := \
	$(shell find $(CLOSURE_LIBRARY_DIR_PATH) -type d -o -type f -name "*.js")


#
# Function for obtaining the input list for the Closure Compiler that correspond
# to the Closure Library.
#
# Test and perf files are excluded from compilation, because some of them may
# contain duplicate symbols and some not correspond to some strict validation
# rules.
#

CLOSURE_LIBRARY_COMPILER_INPUTS = \
	$(call RELATIVE_PATH,$(CURDIR)/$(CLOSURE_LIBRARY_DIR_PATH),$(ROOT_PATH))/closure \
	'!$(call RELATIVE_PATH,$(CURDIR)/$(CLOSURE_LIBRARY_DIR_PATH),$(ROOT_PATH))/**_test.js' \
	'!$(call RELATIVE_PATH,$(CURDIR)/$(CLOSURE_LIBRARY_DIR_PATH),$(ROOT_PATH))/**_perf.js' \


#
# Flags for the Closure Compiler for the different compilation modes.
#

CLOSURE_LIBRARY_COMPILER_FLAGS := \
	--define goog.DISALLOW_TEST_ONLY_CODE=true \
	--define goog.ENABLE_CHROME_APP_SAFE_SCRIPT_LOADING=true \
	--define goog.ENABLE_DEBUG_LOADER=false \
	--define goog.LOAD_MODULE_USING_EVAL=false \
	--define goog.asserts.ENABLE_ASSERTS=true \
	--define goog.debug.LOGGING_ENABLED=true \
	--hide_warnings_for '$(CLOSURE_LIBRARY_RELATIVE_DIR_PATH)' \

ifeq ($(CONFIG),Release)

CLOSURE_LIBRARY_COMPILER_FLAGS += \
	--define goog.DEBUG=false \

else

CLOSURE_LIBRARY_COMPILER_FLAGS += \
	--define goog.DEBUG=true \

endif

CLOSURE_LIBRARY_TESTING_ADDITIONAL_COMPILER_FLAGS := \
	--define goog.DISALLOW_TEST_ONLY_CODE=false \
