// Copyright 2015 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include <memory>
#include <string>
#include <thread>

#include <ppapi/c/ppb_instance.h>
#include <ppapi/cpp/core.h>
#include <ppapi/cpp/instance.h>
#include <ppapi/cpp/module.h>
#include <ppapi/cpp/var.h>

#include <google_smart_card_common/logging/logging.h>
#include <google_smart_card_common/messaging/typed_message_router.h>
#include <google_smart_card_common/pp_var_utils/debug_dump.h>

#include <google_smart_card_libusb_nacl/global.h>

#include "libusb_demo.h"

namespace google_smart_card {

namespace {

class LibusbDemoRunner final : public TypedMessageListener {
 public:
  std::string GetListenedMessageType() const override {
    return "run_libusb_demo";
  }

  bool OnTypedMessageReceived(const pp::Var& /*data*/) override {
    std::thread(RunLibusbDemo).detach();
    return true;
  }
};

class PpInstance final : public pp::Instance {
 public:
  explicit PpInstance(PP_Instance instance)
      : pp::Instance(instance),
        libusb_nacl_global_(new LibusbNaclGlobal(
            &typed_message_router_, this, pp::Module::Get()->core())) {
    typed_message_router_.AddRoute(&libusb_demo_runner_);
  }

  ~PpInstance() {
    libusb_nacl_global_->Detach();
  }

  void HandleMessage(const pp::Var& message) override {
    if (!typed_message_router_.OnMessageReceived(message)) {
      GOOGLE_SMART_CARD_LOG_FATAL << "Unexpected message received: " <<
          DebugDumpVar(message);
    }
  }

 private:
  LibusbDemoRunner libusb_demo_runner_;
  TypedMessageRouter typed_message_router_;
  LibusbNaclGlobal* libusb_nacl_global_;
};

class PpModule final : public pp::Module {
 public:
  pp::Instance* CreateInstance(PP_Instance instance) override {
    return new PpInstance(instance);
  }
};

}  // namespace

}  // namespace google_smart_card

namespace pp {

Module* CreateModule() {
  return new google_smart_card::PpModule();
}

}  // namespace pp
