// Copyright 2015 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

goog.provide('GoogleSmartCard.PcscLiteCcidIntegrationTestApp.WindowMain');

goog.require('GoogleSmartCard.Libusb.ChromeUsbBackend');
goog.require('GoogleSmartCard.NaclModule');
goog.require('goog.dom');
goog.require('goog.events');
goog.require('goog.log');
goog.require('goog.log.Level');
goog.require('goog.log.LogRecord');
goog.require('goog.log.Logger');
goog.require('goog.string');

goog.scope(function() {

/** @const */
var USB_DEVICE_FILTERS = [{'interfaceClass': 0x0B}];

/** @const */
var GSC = GoogleSmartCard;

var naclModule = new GSC.NaclModule(
    'nacl_module.nmf', GSC.NaclModule.Type.PNACL);

var libusbChromeUsbBackend = new GSC.Libusb.ChromeUsbBackend(
    naclModule.messageChannel);

function onSelectDevicesClicked() {
  chrome.usb.getUserSelectedDevices(
      {'multiple': true, 'filters': USB_DEVICE_FILTERS}, goog.nullFunction);
}

function onRunDemoClicked() {
  naclModule.messageChannel.send('run_pcsc_lite_demo', {});
}

/**
 * @param {!goog.log.LogRecord} logRecord
 * @return {boolean}
 */
function isLogMessageForDisplaying(logRecord) {
  if (logRecord.getLevel().value > goog.log.Level.INFO.value)
    return true;
  if (logRecord.getLevel() == goog.log.Level.INFO) {
    if (logRecord.getLoggerName() !=
        naclModule.logMessagesReceiver.logger.getName()) {
      return true;
    }
    return goog.string.startsWith(logRecord.getMessage(), '[PC/SC-Lite DEMO] ');
  }
  return false;
}

/**
 * @param {!goog.log.LogRecord} logRecord
 */
function onLogMessage(logRecord) {
  if (isLogMessageForDisplaying(logRecord)) {
    var element = goog.dom.getElement('debug-log');
    element.appendChild(goog.dom.createTextNode(logRecord.getMessage()));
    element.appendChild(goog.dom.createElement('br'));
  }
}

function onNaclModuleLoaded() {
  goog.dom.getElement('run-demo').disabled = false;
}

var selectDevicesElement = goog.dom.getElement('select-devices');
goog.events.listen(selectDevicesElement, 'click', onSelectDevicesClicked);

goog.events.listen(goog.dom.getElement('run-demo'), 'click', onRunDemoClicked);
naclModule.addLoadEventListener(onNaclModuleLoaded);

naclModule.logger.addHandler(onLogMessage);

naclModule.load();

});  // goog.scope
