// Copyright 2015 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef GOOGLE_SMART_CARD_THIRD_PARTY_PCSC_LITE_COMMON_PCSC_LITE_H_
#define GOOGLE_SMART_CARD_THIRD_PARTY_PCSC_LITE_COMMON_PCSC_LITE_H_

#include <pcsclite.h>
#include <wintypes.h>

namespace google_smart_card {

// Interface corresponding to the PC/SC-Lite API.
//
// All functions presented here have the same signatures as the original
// PC/SC-Lite API functions (see the winscard.h header in the PC/SC-Lite
// sources).
//
// Note that the pcsc_stringify_error function (that in the original
// PC/SC-Lite sources is defined in the pcsclite.h header) is not present here:
// there is no reason to provide a polymorphic behavior to this simple mapping
// function.
class PcscLite {
 public:
  virtual ~PcscLite() = default;

  virtual LONG SCardEstablishContext(
      DWORD dwScope,
      LPCVOID pvReserved1,
      LPCVOID pvReserved2,
      LPSCARDCONTEXT phContext) = 0;

  virtual LONG SCardReleaseContext(SCARDCONTEXT hContext) = 0;

  virtual LONG SCardConnect(
      SCARDCONTEXT hContext,
      LPCSTR szReader,
      DWORD dwShareMode,
      DWORD dwPreferredProtocols,
      LPSCARDHANDLE phCard,
      LPDWORD pdwActiveProtocol) = 0;

  virtual LONG SCardReconnect(
      SCARDHANDLE hCard,
      DWORD dwShareMode,
      DWORD dwPreferredProtocols,
      DWORD dwInitialization,
      LPDWORD pdwActiveProtocol) = 0;

  virtual LONG SCardDisconnect(SCARDHANDLE hCard, DWORD dwDisposition) = 0;

  virtual LONG SCardBeginTransaction(SCARDHANDLE hCard) = 0;

  virtual LONG SCardEndTransaction(SCARDHANDLE hCard, DWORD dwDisposition) = 0;

  virtual LONG SCardStatus(
      SCARDHANDLE hCard,
      LPSTR mszReaderName,
      LPDWORD pcchReaderLen,
      LPDWORD pdwState,
      LPDWORD pdwProtocol,
      LPBYTE pbAtr,
      LPDWORD pcbAtrLen) = 0;

  virtual LONG SCardGetStatusChange(
      SCARDCONTEXT hContext,
      DWORD dwTimeout,
      SCARD_READERSTATE* rgReaderStates,
      DWORD cReaders) = 0;

  virtual LONG SCardControl(
      SCARDHANDLE hCard,
      DWORD dwControlCode,
      LPCVOID pbSendBuffer,
      DWORD cbSendLength,
      LPVOID pbRecvBuffer,
      DWORD cbRecvLength,
      LPDWORD lpBytesReturned) = 0;

  virtual LONG SCardGetAttrib(
      SCARDHANDLE hCard,
      DWORD dwAttrId,
      LPBYTE pbAttr,
      LPDWORD pcbAttrLen) = 0;

  virtual LONG SCardSetAttrib(
      SCARDHANDLE hCard,
      DWORD dwAttrId,
      LPCBYTE pbAttr,
      DWORD cbAttrLen) = 0;

  virtual LONG SCardTransmit(
      SCARDHANDLE hCard,
      const SCARD_IO_REQUEST* pioSendPci,
      LPCBYTE pbSendBuffer,
      DWORD cbSendLength,
      SCARD_IO_REQUEST* pioRecvPci,
      LPBYTE pbRecvBuffer,
      LPDWORD pcbRecvLength) = 0;

  virtual LONG SCardListReaders(
      SCARDCONTEXT hContext,
      LPCSTR mszGroups,
      LPSTR mszReaders,
      LPDWORD pcchReaders) = 0;

  virtual LONG SCardFreeMemory(SCARDCONTEXT hContext, LPCVOID pvMem) = 0;

  virtual LONG SCardListReaderGroups(
      SCARDCONTEXT hContext, LPSTR mszGroups, LPDWORD pcchGroups) = 0;

  virtual LONG SCardCancel(SCARDCONTEXT hContext) = 0;

  virtual LONG SCardIsValidContext(SCARDCONTEXT hContext) = 0;
};

}  // namespace google_smart_card

#endif  // GOOGLE_SMART_CARD_THIRD_PARTY_PCSC_LITE_COMMON_PCSC_LITE_H_
