// Copyright 2015 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// This file contains helper functions for making debug dumps of the PC/SC-Lite
// API values (error codes, bit masks, structures etc.).

#ifndef GOOGLE_SMART_CARD_THIRD_PARTY_PCSC_LITE_COMMON_SCARD_DEBUG_DUMP_H_
#define GOOGLE_SMART_CARD_THIRD_PARTY_PCSC_LITE_COMMON_SCARD_DEBUG_DUMP_H_

#include <stdint.h>

#include <string>

#include <pcsclite.h>
#include <reader.h>
#include <winscard.h>
#include <wintypes.h>

namespace google_smart_card {

std::string DebugDumpSCardReturnCode(LONG return_code);

std::string DebugDumpSCardCString(const char* value);

std::string DebugDumpSCardMultiString(const char* value);

std::string DebugDumpSCardScope(DWORD scope);

std::string DebugDumpSCardShareMode(DWORD share_mode);

std::string DebugDumpSCardProtocol(DWORD protocol);

std::string DebugDumpSCardProtocols(DWORD protocols);

std::string DebugDumpSCardDisposition(DWORD disposition);

std::string DebugDumpSCardState(DWORD state);

std::string DebugDumpSCardEventState(DWORD event_state);

std::string DebugDumpSCardAttributeId(DWORD attribute_id);

std::string DebugDumpSCardControlCode(DWORD control_code);

std::string DebugDumpSCardIoRequest(const SCARD_IO_REQUEST& value);
std::string DebugDumpSCardIoRequest(const SCARD_IO_REQUEST* value);

std::string DebugDumpSCardInputReaderState(const SCARD_READERSTATE& value);
std::string DebugDumpSCardInputReaderStates(
    const SCARD_READERSTATE* begin, DWORD count);
std::string DebugDumpSCardOutputReaderState(const SCARD_READERSTATE& value);
std::string DebugDumpSCardOutputReaderStates(
    const SCARD_READERSTATE* begin, DWORD count);

std::string DebugDumpSCardInputBuffer(const void* buffer, DWORD buffer_size);

std::string DebugDumpSCardBufferSizeInputPointer(const DWORD* buffer_size);

std::string DebugDumpSCardOutputBuffer(
    const void* buffer, const DWORD* buffer_size, bool is_autoallocated);
std::string DebugDumpSCardOutputBuffer(const void* buffer, DWORD buffer_size);

std::string DebugDumpSCardOutputCStringBuffer(
    LPCSTR buffer, bool is_autoallocated);

std::string DebugDumpSCardOutputMultiStringBuffer(
    LPCSTR buffer, bool is_autoallocated);

}  // namespace google_smart_card

#endif  // GOOGLE_SMART_CARD_THIRD_PARTY_PCSC_LITE_COMMON_SCARD_DEBUG_DUMP_H_
