// Copyright 2015 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include <google_smart_card_pcsc_lite_common/scard_structs_serialization.h>

#include <cstring>

#include <google_smart_card_common/pp_var_utils/struct_converter.h>

namespace google_smart_card {

template <>
constexpr const char*
StructConverter<InboundSCardReaderState>::GetStructTypeName() {
  return "SCARD_READERSTATE_inbound";
}

template <>
template <typename Callback>
void StructConverter<InboundSCardReaderState>::VisitFields(
    const InboundSCardReaderState& value, Callback callback) {
  callback(&value.reader_name, "reader_name");
  callback(&value.user_data, "user_data");
  callback(&value.current_state, "current_state");
}

template <>
constexpr const char*
StructConverter<OutboundSCardReaderState>::GetStructTypeName() {
  return "SCARD_READERSTATE_outbound";
}

template <>
template <typename Callback>
void StructConverter<OutboundSCardReaderState>::VisitFields(
    const OutboundSCardReaderState& value, Callback callback) {
  callback(&value.reader_name, "reader_name");
  callback(&value.user_data, "user_data");
  callback(&value.current_state, "current_state");
  callback(&value.event_state, "event_state");
  callback(&value.atr, "atr");
}

template <>
constexpr const char*
StructConverter<SCardIoRequest>::GetStructTypeName() {
  return "SCARD_IO_REQUEST";
}

template <>
template <typename Callback>
void StructConverter<SCardIoRequest>::VisitFields(
    const SCardIoRequest& value, Callback callback) {
  callback(&value.protocol, "protocol");
}

InboundSCardReaderState InboundSCardReaderState::FromSCardReaderState(
    const SCARD_READERSTATE& value) {
  return InboundSCardReaderState(
      value.szReader,
      value.pvUserData ?
          reinterpret_cast<uintptr_t>(value.pvUserData) : optional<uintptr_t>(),
      value.dwCurrentState);
}

OutboundSCardReaderState OutboundSCardReaderState::FromSCardReaderState(
    const SCARD_READERSTATE& value) {
  return OutboundSCardReaderState(
      value.szReader,
      value.pvUserData ?
          reinterpret_cast<uintptr_t>(value.pvUserData) : optional<uintptr_t>(),
      value.dwCurrentState,
      value.dwEventState,
      std::vector<uint8_t>(value.rgbAtr, value.rgbAtr + value.cbAtr));
}

SCARD_IO_REQUEST SCardIoRequest::AsSCardIoRequest() const {
  SCARD_IO_REQUEST result;
  result.dwProtocol = protocol;
  result.cbPciLength = sizeof(SCARD_IO_REQUEST);
  return result;
}

SCardIoRequest SCardIoRequest::FromSCardIoRequest(
    const SCARD_IO_REQUEST& value) {
  return SCardIoRequest(value.dwProtocol);
}

template <>
pp::Var MakeVar(const SCARD_READERSTATE& value) {
  VarDictBuilder result_builder;
  result_builder.Add("reader_name", value.szReader);
  if (value.pvUserData) {
    result_builder.Add(
        "user_data", reinterpret_cast<uintptr_t>(value.pvUserData));
  }
  result_builder.Add("current_state", value.dwCurrentState);
  result_builder.Add("event_state", value.dwEventState);
  // Chrome Extensions API does not allow sending ArrayBuffers in message
  // fields, so instead of pp::VarArrayBuffer a pp::VarArray with the bytes as
  // its element is constructed.
  result_builder.Add("atr", MakeVar(std::vector<uint8_t>(
      value.rgbAtr, value.rgbAtr + value.cbAtr)));
  return result_builder.Result();
}

bool VarAs(
    const pp::Var& var,
    InboundSCardReaderState* result,
    std::string* error_message) {
  return StructConverter<InboundSCardReaderState>::ConvertFromVar(
      var, result, error_message);
}

template <>
pp::Var MakeVar(const InboundSCardReaderState& value) {
  return StructConverter<InboundSCardReaderState>::ConvertToVar(value);
}

bool VarAs(
    const pp::Var& var,
    OutboundSCardReaderState* result,
    std::string* error_message) {
  return StructConverter<OutboundSCardReaderState>::ConvertFromVar(
      var, result, error_message);
}

template <>
pp::Var MakeVar(const OutboundSCardReaderState& value) {
  return StructConverter<OutboundSCardReaderState>::ConvertToVar(value);
}

bool VarAs(
    const pp::Var& var, SCardIoRequest* result, std::string* error_message) {
  return StructConverter<SCardIoRequest>::ConvertFromVar(
      var, result, error_message);
}

template <>
pp::Var MakeVar(const SCardIoRequest& value) {
  return StructConverter<SCardIoRequest>::ConvertToVar(value);
}

}  // namespace google_smart_card
