// Copyright 2015 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include <cstring>
#include <string>

extern "C" {
#include "config.h"
#include "pcsclite.h"
#include "dyn_generic.h"
#include "ifdhandler.h"
#include "misc.h"

// "misc.h" defines a "min" macro which is incompatible with C++.
#undef min
}

#include <google_smart_card_common/logging/logging.h>

// This is a fake value that we supply instead of the dynamically loaded library
// handles which are used by pcsc-lite normally.
const char kFakeLHandle[] = "fake_pcsclite_lhandle";

namespace {

struct FunctionNameAndAddress {
  const char* name;
  void* address;
};

}  // namespace

// Fake export table of the driver functions (the function pointers here point
// directly to the statically linked driver functions).
const FunctionNameAndAddress kDriverIfdhandlerFunctions[] = {
  {"IFDHCreateChannelByName",
   reinterpret_cast<void*>(&IFDHCreateChannelByName)},
  {"IFDHCreateChannel",
   reinterpret_cast<void*>(&IFDHCreateChannel)},
  {"IFDHCloseChannel",
   reinterpret_cast<void*>(&IFDHCloseChannel)},
  {"IFDHGetCapabilities",
   reinterpret_cast<void*>(&IFDHGetCapabilities)},
  {"IFDHSetCapabilities",
   reinterpret_cast<void*>(&IFDHSetCapabilities)},
  {"IFDHSetProtocolParameters",
   reinterpret_cast<void*>(&IFDHSetProtocolParameters)},
  {"IFDHPowerICC",
   reinterpret_cast<void*>(&IFDHPowerICC)},
  {"IFDHTransmitToICC",
   reinterpret_cast<void*>(&IFDHTransmitToICC)},
  {"IFDHControl",
   reinterpret_cast<void*>(&IFDHControl)},
  {"IFDHICCPresence",
   reinterpret_cast<void*>(&IFDHICCPresence)},
};

INTERNAL int DYN_LoadLibrary(void** pvLHandle, char* pcLibrary) {
  *pvLHandle = static_cast<void*>(const_cast<char*>(kFakeLHandle));
  return SCARD_S_SUCCESS;
}

INTERNAL int DYN_CloseLibrary(void** pvLHandle) {
  GOOGLE_SMART_CARD_CHECK(*pvLHandle == kFakeLHandle);
  *pvLHandle = nullptr;
  return SCARD_S_SUCCESS;
}

INTERNAL int DYN_GetAddress(
    void* pvLHandle,
    void** pvFHandle,
    const char* pcFunction,
    int /*mayfail*/) {
  GOOGLE_SMART_CARD_CHECK(pvLHandle == kFakeLHandle);
  for (const FunctionNameAndAddress& driver_ifdhandler_function
       : kDriverIfdhandlerFunctions) {
    if (std::string(pcFunction) == driver_ifdhandler_function.name) {
      *pvFHandle = driver_ifdhandler_function.address;
      return SCARD_S_SUCCESS;
    }
  }
  *pvFHandle = NULL;
  return SCARD_F_UNKNOWN_ERROR;
}
