// Copyright 2015 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include "server_sockets_manager.h"

#include <google_smart_card_common/logging/logging.h>

namespace google_smart_card {

namespace {

PcscLiteServerSocketsManager* g_pcsc_lite_server_sockets_manager = nullptr;

}  // namespace

void PcscLiteServerSocketsManager::CreateGlobalInstance() {
  GOOGLE_SMART_CARD_CHECK(!g_pcsc_lite_server_sockets_manager);
  g_pcsc_lite_server_sockets_manager = new PcscLiteServerSocketsManager;
}

PcscLiteServerSocketsManager* PcscLiteServerSocketsManager::GetInstance() {
  GOOGLE_SMART_CARD_CHECK(g_pcsc_lite_server_sockets_manager);
  return g_pcsc_lite_server_sockets_manager;
}

void PcscLiteServerSocketsManager::Push(int server_socket_file_descriptor) {
  const std::unique_lock<std::mutex> lock(mutex_);
  server_socket_file_descriptors_queue_.push(server_socket_file_descriptor);
  condition_.notify_all();
}

int PcscLiteServerSocketsManager::WaitAndPop() {
  std::unique_lock<std::mutex> lock(mutex_);
  condition_.wait(
      lock,
      [this]() {
        return !server_socket_file_descriptors_queue_.empty();
      });
  int result = server_socket_file_descriptors_queue_.front();
  server_socket_file_descriptors_queue_.pop();
  return result;
}

}  // namespace google_smart_card
