// Copyright 2015 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef GOOGLE_SMART_CARD_THIRD_PARTY_PCSC_LITE_SERVER_SOCKETS_MANAGER_H_
#define GOOGLE_SMART_CARD_THIRD_PARTY_PCSC_LITE_SERVER_SOCKETS_MANAGER_H_

#include <condition_variable>
#include <mutex>
#include <queue>

namespace google_smart_card {

// Holder of a queue of server-side sockets for the socket pairs created at the
// client side.
class PcscLiteServerSocketsManager final {
 public:
  // Note: This function is not thread-safe!
  static void CreateGlobalInstance();
  // Note: This function is not thread-safe!
  static PcscLiteServerSocketsManager* GetInstance();

  void Push(int server_socket_file_descriptor);

  int WaitAndPop();

 private:
  std::mutex mutex_;
  std::condition_variable condition_;
  std::queue<int> server_socket_file_descriptors_queue_;
};

}  // namespace google_smart_card

#endif  // GOOGLE_SMART_CARD_THIRD_PARTY_PCSC_LITE_SERVER_SOCKETS_MANAGER_H_
