// Copyright 2015 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef GOOGLE_SMART_CARD_THIRD_PARTY_PCSC_LITE_CLIENT_HANDLES_REGISTRY_H_
#define GOOGLE_SMART_CARD_THIRD_PARTY_PCSC_LITE_CLIENT_HANDLES_REGISTRY_H_

#include <mutex>
#include <unordered_set>

#include <pcsclite.h>
#include <reader.h>
#include <winscard.h>
#include <wintypes.h>

#include <google_smart_card_common/logging/logging.h>

namespace google_smart_card {

// FIXME(emaxx): ???
template <typename T>
class PcscLiteClientHandlesRegistry final {
 public:
  bool Contains(T handle) const {
    const std::unique_lock<std::mutex> lock(mutex_);
    return handles_.count(handle) > 0;
  }

  void Add(T handle) {
    const std::unique_lock<std::mutex> lock(mutex_);
    GOOGLE_SMART_CARD_CHECK(handles_.insert(handle).second);
  }

  void Remove(T handle) {
    const std::unique_lock<std::mutex> lock(mutex_);
    GOOGLE_SMART_CARD_CHECK(handles_.erase(handle));
  }

  std::vector<T> PopAll() {
    const std::unique_lock<std::mutex> lock(mutex_);
    std::vector<T> result(handles_.begin(), handles_.end());
    handles_.clear();
    return result;
  }

 private:
  mutable std::mutex mutex_;
  std::unordered_set<T> handles_;
};

using PcscLiteClientSCardContextsRegistry =
    PcscLiteClientHandlesRegistry<SCARDCONTEXT>;

using PcscLiteClientSCardHandlesRegistry =
    PcscLiteClientHandlesRegistry<SCARDHANDLE>;

}  // namespace google_smart_card

#endif  // GOOGLE_SMART_CARD_THIRD_PARTY_PCSC_LITE_CLIENT_HANDLES_REGISTRY_H_
