// Copyright 2015 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef GOOGLE_SMART_CARD_PCSC_LITE_SERVER_CLIENTS_MANAGEMENT_BACKEND_H_
#define GOOGLE_SMART_CARD_PCSC_LITE_SERVER_CLIENTS_MANAGEMENT_BACKEND_H_

#include <memory>

#include <ppapi/cpp/instance.h>

#include <google_smart_card_common/messaging/typed_message_router.h>

namespace google_smart_card {

// The class that enables the PC/SC-Lite server clients management (i.e.
// adding/removing of PC/SC-Lite clients and performing PC/SC-Lite requests
// received from them).
//
// The clients management is enabled in the constructor by adding several typed
// message handlers into the passed typed message router.
//
// Note: clients of the class should ensure that no corresponding incoming
// messages are arrived simultaneously with the class destruction.
class PcscLiteServerClientsManagementBackend final {
 public:
  PcscLiteServerClientsManagementBackend(
      pp::Instance* pp_instance, TypedMessageRouter* typed_message_router);

  ~PcscLiteServerClientsManagementBackend();

 private:
  class Impl;

  std::unique_ptr<Impl> impl_;
};

}  // namespace google_smart_card

#endif  // GOOGLE_SMART_CARD_PCSC_LITE_SERVER_CLIENTS_MANAGEMENT_BACKEND_H_
