// Copyright 2015 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

goog.provide('GoogleSmartCard.PcscLiteServerClientsManagement.PermissionsChecking.Checker');

goog.require('GoogleSmartCard.DebugDump');
goog.require('GoogleSmartCard.Logging');
goog.require('GoogleSmartCard.PcscLiteServerClientsManagement.PermissionsChecking.ManagedRegistry');
goog.require('GoogleSmartCard.PcscLiteServerClientsManagement.PermissionsChecking.UserPromptingChecker');
goog.require('goog.Promise');
goog.require('goog.log.Logger');
goog.require('goog.promise.Resolver');

goog.scope(function() {

/** @const */
var GSC = GoogleSmartCard;

/** @const */
var PermissionsChecking =
    GSC.PcscLiteServerClientsManagement.PermissionsChecking;

/**
 * FIXME(emaxx)
 * @constructor
 */
PermissionsChecking.Checker = function() {
  /** @private */
  this.managedRegistry_ = new PermissionsChecking.ManagedRegistry;
  /** @private */
  this.userPromptingChecker_ = new PermissionsChecking.UserPromptingChecker;
};

/** @const */
var Checker = PermissionsChecking.Checker;

/**
 * @type {!goog.log.Logger}
 * @const
 */
Checker.prototype.logger = GSC.Logging.getScopedLogger(
    'PcscLiteServerClientsManagement.PermissionsChecking.Checker');

/**
 * FIXME(emaxx)
 * @param {string|null} clientAppId
 * @return {!goog.Promise}
 */
Checker.prototype.check = function(clientAppId) {
  this.logger.finer('Checking permissions for client App with id ' +
                    GSC.DebugDump.dump(clientAppId) + '...');

  if (goog.isNull(clientAppId)) {
    this.logger.finer('Granted permissions for client with null App id');
    return goog.Promise.resolve();
  }

  var checkPromiseResolver = goog.Promise.withResolver();

  this.checkByManagedRegistry_(clientAppId, checkPromiseResolver);

  return checkPromiseResolver.promise;
};

/**
 * @param {string} clientAppId
 * @param {!goog.promise.Resolver} checkPromiseResolver
 * @private
 */
Checker.prototype.checkByManagedRegistry_ = function(
    clientAppId, checkPromiseResolver) {
  this.logger.finer('Checking permissions for the client App with id "' +
                    clientAppId + '" through the managed registry...');

  this.managedRegistry_.getById(clientAppId).then(
      function() {
        this.logger.finer('Granted permissions for client App with id "' +
                          clientAppId + '" through the managed registry');
        checkPromiseResolver.resolve();
      },
      function() {
        this.logger.finer('No permissions found for client App with id "' +
                          clientAppId + '" through the managed registry');
        this.checkByUserPromptingChecker_(clientAppId, checkPromiseResolver);
      },
      this);
};

/**
 * @param {string} clientAppId
 * @param {!goog.promise.Resolver} checkPromiseResolver
 * @private
 */
Checker.prototype.checkByUserPromptingChecker_ = function(
    clientAppId, checkPromiseResolver) {
  this.userPromptingChecker_.check(clientAppId).then(
      checkPromiseResolver.resolve, checkPromiseResolver.reject);
};

});  // goog.scope
