/**
 * @fileoverview Main script of the PC/SC-Lite server client permissions user
 * prompt dialog window.
 *
 * Once the dialog is finished with some result (either user pressed the allow
 * button or the deny button, or the dialog was canceled), it uses the methods
 * of the GoogleSmartCard.ModalDialog library for returning the result to the
 * caller window (i.e. to the App's background page).
 */

goog.provide('GoogleSmartCard.PcscLiteServerClientsManagement.PermissionsChecking.UserPromptDialog.Main');

goog.require('GoogleSmartCard.Logging');
goog.require('GoogleSmartCard.ModalDialog.Client');
goog.require('goog.dom');
goog.require('goog.dom.forms');
goog.require('goog.events');
goog.require('goog.log.Logger');

goog.scope(function() {

/** @const */
var GSC = GoogleSmartCard;

/**
 * @type {!goog.log.Logger}
 * @const
 */
var logger = GSC.Logging.getScopedLogger(
    'PcscLiteServerClientsManagement.PermissionsChecking.UserPromptDialog.' +
    'Main');

function displayMessage() {
  var data = GSC.ModalDialog.Client.getData();

  var clientAppId = data['client_app_id'];
  GSC.Logging.checkWithLogger(logger, goog.isString(clientAppId));
  goog.asserts.assertString(clientAppId);

  var clientAppName = data['client_app_name'];
  GSC.Logging.checkWithLogger(
      logger, !goog.isDef(clientAppName) || goog.isString(clientAppName));
  goog.asserts.assert(
      !goog.isDef(clientAppName) || goog.isString(clientAppName));
  if (!goog.DEBUG) {
    GSC.Logging.checkWithLogger(logger, goog.isDef(clientAppName));
    goog.asserts.assert(goog.isDef(clientAppName));
  }

  var message;
  if (goog.isDef(clientAppName)) {
    message = chrome.i18n.getMessage(
        'pcscClientHandlingUserPromptDialogMessageForKnownApp',
        [clientAppId, clientAppName]);
  } else {
    message = chrome.i18n.getMessage(
        'pcscClientHandlingUserPromptDialogMessageForUnknownApp', clientAppId);
  }

  goog.dom.setTextContent(goog.dom.getElement('message'), message);
}

function allowClickListener() {
  GSC.ModalDialog.Client.resolveModalDialog(true);
}

function denyClickListener() {
  GSC.ModalDialog.Client.resolveModalDialog(false);
}

displayMessage();

goog.events.listen(goog.dom.getElement('allow'), 'click', allowClickListener);
goog.events.listen(goog.dom.getElement('deny'), 'click', denyClickListener);

GSC.ModalDialog.Client.prepareAndShowModalDialog();

});  // goog.scope
